#include <stdbool.h>
#include <stdint.h>

#ifndef IMPULS_COUNTER_H
#define IMPULS_COUNTER_H

#ifndef IRAM_ATTR
    #define IRAM_ATTR __attribute__((section(".iram.text")))
#endif

struct impuls_measurement {
    uint32_t samples; /* number of samples */
    uint32_t changes; /* input state changes */
};

struct impuls_counter {
    bool input_initialized; /* false before first update call */
    bool input_state; /* last update input state */

    uint32_t counter; /* input state changes counter */
    uint32_t samples; /* input samples counter */
    uint32_t cycle_len; /* minimum number of samples in one cycle */

    bool new_measurement; /* set to true after updating measurement */
    struct impuls_measurement measurement; /* one measurement */
};

/**
 * @brief initialize impuls counter structure
 *
 * @param cycle_len number of samples in one measurement cycle
 */
void impuls_counter_init ( struct impuls_counter *f, uint32_t cycle_len );

void IRAM_ATTR impuls_counter_update ( struct impuls_counter *f, bool input );

/** 
 * @brief get new measurement from impuls_counter
 *  
 * this function should be called at least twice per cycle in order to avoid
 * concurrent rw acces to measurement variable
 * @param measurement is set to new measurement value if returned true
 * @return true if new measurement is available
 */
bool impuls_counter_get_measurement( struct impuls_counter *f,
    struct impuls_measurement *measurement);

#endif /* end of include guard: IMPULS_COUNTER_H */
